{-

DFA simulator in simple but idiomatic Haskell.
Illustrative example: a(ba)*

-}

module Dfa1 where

import Data.Maybe


-- Representation of a DFA

-- BEGIN_DFA_TYPES
type State = Int
type Label = Char
type Transitions = [(State, [(Label, State)])]
type Accept = [State]
-- END_DFA_TYPES

-- Run the input on a DFA and report acceptance

-- BEGIN_HASKELL_DFA_RUN
run :: State -> Accept -> Transitions -> [Label] -> Bool
run current accept transitions [] = current `elem` accept
run current accept transitions (l:ls) =
  case lookup l (fromJust (lookup current transitions)) of
    Nothing -> False
    Just next -> run next accept transitions ls
-- END_HASKELL_DFA_RUN


-- Final states and transition function for an example

-- BEGIN_DFA_MACHINE
transitions = [(1, [('a', 2)]), 
                    (2, [('b', 1)])]
accept = [2]
-- END_DFA_MACHINE
  

-- Testing

main
 = do
      print $ run 1 accept transitions ['a']
      print $ run 1 accept transitions ['b']
      print $ run 1 accept transitions ['a', 'b']
      print $ run 1 accept transitions ['a', 'b', 'a']
